/*
	Dragon disk operations
*/

#include <stddef.h>
#include <bios.h>
#include <dos.h>

#include "global.h"
#include "hookdbt.h"

#define SIMPLE_REGS union REGS
#define SIMPLE_INT(r) int86 (0x13, &(r), &(r))

/* prototypes */

void reset_drive(int drive);
const char *bios_error_description(int error_code);


/* function defs */

int bios_get_drive_type(int drive)
{
	Disk_Base_Table	dbt;             /* returns:         */
	union REGS regs;                 /* 0 = no drive     */
									 /* 1 = 360K   5.25  */
	regs.h.ah = 0x08;                /* 2 = 1.2M   5.25  */
	regs.h.dl = drive;               /* 3 = 720K   3.5   */
									 /* 4 = 1.44M  3.5   */
	int86(0x13, &regs, &regs);

	return regs.h.bl;
}


int read_sector(int drive_num, int track, int side, int sector, Byte *buf)
{
	Disk_Base_Table	dbt;
	int	result, count = 0;

	hook_disk_base_table(&dbt);
	dbt.table[3] = 0x01;
	dbt.table[4] = 18;

	while(count++ < 3)
	{
		result = biosdisk(_DISK_READ, drive_num, side, track, sector, 1, buf);

		if(!result)
			break;

		reset_drive(drive_num);
	}

	unhook_disk_base_table(&dbt);

	return result;
}


int write_sector(int drive_num, int track, int side, int sector, Byte *buf)
{
	Disk_Base_Table	dbt;
	int	result, count = 0;

	hook_disk_base_table(&dbt);
	dbt.table[3] = 0x01;
	dbt.table[4] = 18;

	while(count++ < 3)
	{
		result = biosdisk(_DISK_WRITE, drive_num, side, track, sector, 1, buf);

		if(!result || (result==3))
			break;

		reset_drive(drive_num);
	}

	unhook_disk_base_table(&dbt);

	return result;
}


void reset_drive(int drive)
{
	biosdisk(_DISK_RESET, drive, 0, 0, 0, 0, NULL);
}


int format_track(int drive, int track, int side)
{
	Disk_Base_Table	dbt;
	int	result, count = 0, sector;
	Byte buf[18*4];
	Byte *ptr = buf;

	/* set up sector addresses with standard interleave */
	for(sector = 1; sector < 10; sector++)
	{
		*ptr++ = track;
		*ptr++ = side;
		*ptr++ = sector;
		*ptr++ = 1;

		*ptr++ = track;
		*ptr++ = side;
		*ptr++ = sector+9;
		*ptr++ = 1;
	}

	hook_disk_base_table (&dbt);

	dbt.table[3] = 0x01;
	dbt.table[4] = 18;
	dbt.table[7] = 24;

	while (count++ < 3)
	{

		result = biosdisk(_DISK_FORMAT, drive, side, track, 1, 1, buf);

		if(!result || (result==3)) break;

		reset_drive(drive);
	}

	unhook_disk_base_table (&dbt);

	return result;
}


int verify_sectors(int drive, int track, int side, int start_sec, int num_sec)
{
	Disk_Base_Table	dbt;
	int result,	count = 0;

	hook_disk_base_table(&dbt);
	dbt.table[3] = 0x01;
	dbt.table[4] = 18;

	while(count++ < 1)
	{
		result = biosdisk(_DISK_VERIFY, drive, side, track, start_sec, num_sec, NULL);

		if(!result) break;

		reset_drive(drive);
	}

	unhook_disk_base_table(&dbt);

	return result;
}


int bios_set_dasd_type (int drive, int dasd_type)
{
	SIMPLE_REGS	regs;

	regs.h.ah = 0x17;		/*  Int 13h AH=17h, set DASD type for format  */
	regs.h.dl = drive;		/*  DL = drive number (0-3)                   */
	regs.h.al = dasd_type;	/*  AL = DASD type: 0x01  360K in 360K
							 *                  0x02  360K in 1.2M
							 *                  0x03  1.2M in 1.2M
							 *                  0x04  720K in 720K or 1.44M
							 */
	SIMPLE_INT(regs);

	if(regs.h.ah)
		reset_drive(drive);

	return(regs.h.ah);
}


int bios_set_media_type(int drive, int max_track, int max_sector)
{
	SIMPLE_REGS regs;

	regs.h.ah = 0x18;		/*  Int 13h AH=18h, set media type for format  */
	regs.h.dl = drive;		/*  DL = drive number (0-3)                    */
	regs.h.ch = max_track;	/*  CH = max cylinder (num cyls - 1)           */
	regs.h.cl = max_sector;	/*  CL = max sector                            */

	SIMPLE_INT(regs);

	if(regs.h.ah)
		reset_drive(drive);

	return(regs.h.ah);
}


typedef struct bios_error_entry_struct
{
	const Byte			code;
	const char * const	description;
} BIOS_Error_Entry;


static BIOS_Error_Entry	bios_error_table [] =
{
	{ 0x00,	"No Error" },
	{ 0x01,	"Illegal command to driver" },
	{ 0x02,	"Address mark not found" },
	{ 0x03,	"Disk is write-protected" },
	{ 0x04,	"Sector not found" },
	{ 0x06,	"Disk change not caught" },
	{ 0x08,	"DMA overrun" },
	{ 0x09,	"DMA across 64k page boundary" },
	{ 0x0c,	"Invalid media" },
	{ 0x10,	"CRC error on disk read" },
	{ 0x20,	"Controller error" },
	{ 0x40,	"Seek failure" },
	{ 0x80,	"Timeout - drive not ready" },
	{ 0xff,	NULL }
};


const char *bios_error_description(int error_code)
{
	BIOS_Error_Entry	*error_ptr;

	for (error_ptr = bios_error_table; error_ptr->description != NULL; error_ptr++)
		if (error_code == error_ptr->code)
			return (error_ptr->description);

	return ((const char *) "Internal error");
}

